#コミック
class Comic < ActiveRecord::Base
  has_many :stories
  belongs_to :author
  
  validates :title, :presence => true, :length => {:maximum => 100}
  validates :visible, :presence => true, :numericality => true, :inclusion => {:in => 0..3}
  
  before_save do |r|
    r.supply_default
  end
  
  def supply_default
    self.visible = 0 if self.visible.blank?
  end
  
  def overwrite au
    return false unless au
    self.author_id = au.id
  end
  
  def own? author
    return false unless author
    self.author_id == author.id
  end
  
  def visible? author
    self.visible > 0 or self.own?(author)
  end
  
  def disp_visible
    visible == 1 ? 'O' : 'X'
  end
  
  def self.default_page_size
    25
  end
  
  def self.max_page_size
    100
  end
  
  def self.default_panel_size
    30
  end
  
  def self.max_panel_size
    200
  end
  
  def self.page prm = nil
    page = prm.to_i
    page = 1 if page < 1
    page
  end
  
  def self.page_size prm = self.default_page_size
    page_size = prm.to_i
    page_size = self.max_page_size if page_size > self.max_page_size
    page_size = self.default_page_size if page_size < 1
    page_size
  end
  
  def self.list opt = {}, page = 1, page_size = self.default_page_size
    opt.merge!(self.list_opt) unless opt[:include]
    opt.merge!({:conditions => ['visible > 0'], :order => 'updated_at desc', :limit => page_size, :offset => (page -1) * page_size})
    Comic.find(:all, opt)
  end
  
  def self.list_opt
    {:include => {:stories => {:panel => {}}, :author => {}}}
  end
  
  def self.list_json_opt
    {:include => {:stories => {:panel => {}}, :author => {}}}
  end
  
  def self.mylist au, opt = {}, page = 1, comic_page_size = Author.default_comic_page_size
    opt.merge!(self.list_opt) unless opt[:include]
    opt.merge!({:conditions => ['comics.author_id = ?', au.id], :order => 'comics.updated_at desc', :limit => page_size, :offset => (page -1) * comic_page_size})
    Comic.find(:all, opt)
  end
  
  def self.show cid, au, opt = {}
    c = Comic.find(cid, :include => self.show_include_opt(opt))
    raise ActiveRecord::Forbidden unless c.visible?(au)
    c
  end
  
  def self.edit cid, au, opt = {}
    c = Comic.find(cid, :include => self.show_include_opt(opt))
    raise ActiveRecord::Forbidden unless c.own?(au)
    c
  end
  
  def self.show_include_opt opt = {}
    res = {:stories => {:panel => {}}, :author => {}}
    res.push(opt[:include]) if opt[:include]
    res
  end
  
  def self.show_json_include_opt
    {:include => {:stories => {:panel => {}}, :author => {}}}
  end
  
  def self.visible_count
    Comic.count 'visible > 0'
  end
  
end
