# -*- encoding: utf-8 -*-
require 'spec_helper'
#コマの背景間接

describe GroundColor do
  before do
    FactoryGirl.create :admin
    @user = FactoryGirl.create( :user_yas)
    @author = @user.author
    @artist = FactoryGirl.create :artist_yas, :author_id => @author.id
    @other_user = FactoryGirl.create( :user_yas)
    @other_author = @other_user.author
    @other_artist = FactoryGirl.create :artist_yas, :author_id => @other_author.id
    @sp = FactoryGirl.create :system_picture
    @lg = FactoryGirl.create :license_group
    @license = FactoryGirl.create :license, :license_group_id => @lg.id, :system_picture_id => @sp.id
    @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
    @p = FactoryGirl.create :picture, :original_picture_id => @op.id, :license_id => @license.id, :artist_id => @artist.id
    @rp = FactoryGirl.create :resource_picture, :artist_id => @artist.id, :license_id => @license.id, :original_picture_id => @op.id, :picture_id => @p.id
    @panel = FactoryGirl.create :panel, :author_id => @author.id
    @color = FactoryGirl.create :color
  end
  
  describe '検証に於いて' do
    before do
      @gc = FactoryGirl.build :ground_color, :panel_id => @panel.id, :color_id => @color.id
    end
    it 'オーソドックスなデータなら通る' do
      @gc.save
      @gc.should be_valid
    end
    
    context 'panel_idを検証するとき' do
      #ネストの保存はnilを許可しなければならないので数値チェックだけ
      it 'テストデータの確認' do
        @gc.panel_id = @panel.id
        @gc.should be_valid
      end
      it '数値でなければ失敗する' do
        @gc.panel_id = 'a'
        @gc.should_not be_valid
      end
    end
    context 'color_idを検証するとき' do
      it 'テストデータの確認' do
        @gc.color_id = @color.id
        @gc.should be_valid
      end
      it 'nullなら失敗する' do
        @gc.color_id = nil
        @gc.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @gc.color_id = 'a'
        @gc.should_not be_valid
      end
      it '存在する実素材でなければ失敗する' do
        @gc.color_id = 0
        @gc.should_not be_valid
      end
    end
    context 'zを検証するとき' do
      it 'テストデータの確認' do
        @gc.z = 1
        @gc.should be_valid
      end
      it 'nullなら失敗する' do
        @gc.z = nil
        @gc.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @gc.z = 'a'
        @gc.should_not be_valid
      end
      it '負なら失敗する' do
        @gc.z = -1
        @gc.should_not be_valid
      end
      it '負なら失敗する' do
        @gc.z = 0
        @gc.should_not be_valid
      end
    end
  end
  describe '一覧取得に於いて' do
    before do
      @gc = FactoryGirl.create :ground_color, :panel_id => @panel.id, :color_id => @color.id
    end
    context 'page補正について' do
      it '文字列から数値に変換される' do
        GroundColor.page('8').should eq 8
      end
      it 'nilの場合は1になる' do
        GroundColor.page().should eq 1
      end
      it '0以下の場合は1になる' do
        GroundColor.page('0').should eq 1
      end
    end
    context 'page_size補正について' do
      it '文字列から数値に変換される' do
        GroundColor.page_size('7').should eq 7
      end
      it 'nilの場合はGroundColor.default_page_sizeになる' do
        GroundColor.page_size().should eq GroundColor.default_page_size
      end
      it '0以下の場合はGroundColor.default_page_sizeになる' do
        GroundColor.page_size('0').should eq GroundColor.default_page_size
      end
      it 'GroundColor.max_page_sizeを超えた場合はGroundColor.max_page_sizeになる' do
        GroundColor.page_size('1000').should eq GroundColor.max_page_size
      end
    end
    it 'リストを返す' do
      pl = GroundColor.list
      pl.should eq [@gc]
    end
    it '時系列で並んでいる' do
      npl = FactoryGirl.create :ground_color, :panel_id => @panel.id, :color_id => @color.id, :z => 2, :updated_at => Time.now + 100
      pl = GroundColor.list
      pl.should eq [npl, @gc]
    end
    it '非公開のコマの景色は含まない' do
      hpl = FactoryGirl.create :panel, :author_id => @author.id, :publish => 0
      npl = FactoryGirl.create :ground_color, :panel_id => hpl.id, :color_id => @color.id, :updated_at => Time.now + 100
      pl = GroundColor.list
      pl.should eq [@gc]
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        @npl2 = FactoryGirl.create :ground_color, :panel_id => @panel.id, :z => 1, :color_id => @color.id, :updated_at => Time.now + 100
        @npl3 = FactoryGirl.create :ground_color, :panel_id => @panel.id, :z => 2, :color_id => @color.id, :updated_at => Time.now + 200
        @npl4 = FactoryGirl.create :ground_color, :panel_id => @panel.id, :z => 3, :color_id => @color.id, :updated_at => Time.now + 300
        @npl5 = FactoryGirl.create :ground_color, :panel_id => @panel.id, :z => 4, :color_id => @color.id, :updated_at => Time.now + 400
        GroundColor.stub(:default_page_size).and_return(2)
      end
      it '通常は2件を返す' do
        pl = GroundColor.list
        pl.should have(2).items 
      end
      it 'page=1なら末尾2件を返す' do
        #時系列で並んでいる
        pl = GroundColor.list( {}, 1)
        pl.should eq [@npl5, @npl4]
      end
      it 'page=2なら中間2件を返す' do
        pl = GroundColor.list({}, 2)
        pl.should eq [@npl3, @npl2]
      end
      it 'page=3なら先頭1件を返す' do
        pl = GroundColor.list({}, 3)
        pl.should eq [@gc]
      end
    end
  end
  describe 'list関連テーブルプションに於いて' do
    it 'includeキーを含んでいる' do
      r = GroundColor.list_opt
      r.has_key?(:include).should be_true
    end
    it '2つの項目を含んでいる' do
      r = GroundColor.list_opt[:include]
      r.should have(2).items
    end
    it 'コマを含んでいる' do
      r = GroundColor.list_opt[:include]
      r.has_key?(:panel).should be_true
    end
      it 'コマは作家を含んでいる' do
        r = GroundColor.list_opt[:include]
        r[:panel].has_key?(:author).should be_true
      end
    it '色を含んでいる' do
      r = GroundColor.list_opt[:include]
      r.has_key?(:color).should be_true
    end
  end
  describe 'json一覧出力オプションに於いて' do
    it 'includeキーを含んでいる' do
      r = GroundColor.list_json_opt
      r.has_key?(:include).should be_true
    end
    it '2つの項目を含んでいる' do
      r = GroundColor.list_json_opt[:include]
      r.should have(2).items
    end
    it 'コマを含んでいる' do
      r = GroundColor.list_json_opt[:include]
      r.has_key?(:panel).should be_true
    end
      it 'コマは作家を含んでいる' do
        r = GroundColor.list_json_opt[:include]
        r[:panel].has_key?(:author).should be_true
      end
    it '色を含んでいる' do
      r = GroundColor.list_json_opt[:include]
      r.has_key?(:color).should be_true
    end
  end
  
  describe '自分のコマで使った色一覧取得に於いて' do
    before do
      @gc = FactoryGirl.create :ground_color, :panel_id => @panel.id, :color_id => @color.id
    end
    it 'リストを返す' do
      pl = GroundColor.mylist @author
      pl.should eq [@gc]
    end
    it '時系列で並んでいる' do
      npl = FactoryGirl.create :ground_color, :panel_id => @panel.id, :color_id => @color.id, :z => 2, :updated_at => Time.now + 100
      pl = GroundColor.mylist @author
      pl.should eq [npl, @gc]
    end
    it '他人のコマの景色は公開でも含まない' do
      hpl = FactoryGirl.create :panel, :author_id => @other_author.id, :publish => 1
      npl = FactoryGirl.create :ground_color, :panel_id => hpl.id, :color_id => @color.id
      pl = GroundColor.mylist @author
      pl.should eq [@gc]
    end
  end
  
end
