# -*- encoding: utf-8 -*-
require 'spec_helper'
#フキダシテンプレート
describe SpeechBalloonTemplate do
  before do
    #テストデータを用意してね
    @f = Rails.root + 'spec/json/speech_balloon_template.json'
    @t = File.open(@f, 'r').read
    @j = JSON.parse @t
    @fs = Rails.root + 'spec/json/speech_balloon_templates.json'
    @ts = File.open(@fs, 'r').read
    @js = JSON.parse @ts
    @fes = Rails.root + 'spec/json/invalid_speech_balloon_templates.json'
    @tes = File.open(@fes, 'r').read
    @jes = JSON.parse @tes
    
    @admin = FactoryGirl.create :admin
  end
  
  describe '検証に於いて' do
    before do
    end
    
    it 'オーソドックスなデータなら通る' do
      @st = FactoryGirl.build :speech_balloon_template
      @st.should be_valid
    end
    
    context 'nameを検証するとき' do
      before do
        @st = FactoryGirl.build :speech_balloon_template
      end
      it 'テストデータの確認' do
        @st.name = 'abcdefghi0abcdefghi0abcdefghi0abcdefghi0abcdefghi0'
        @st.should be_valid
      end
      it 'nullなら失敗する' do
        @st.name = ''
        @st.should_not be_valid
      end
      it '51文字以上なら失敗する' do
        @st.name = 'a'*51
        @st.should_not be_valid
      end
    end
    context 'classnameを検証するとき' do
      before do
        @st = FactoryGirl.build :speech_balloon_template
      end
      it 'テストデータの確認' do
        @st.classname = 'abcdefghi0abcdefghi0abcdefghi0abcdefghi0abcdefghi0'
        @st.should be_valid
      end
      it 'nullなら失敗する' do
        @st.classname = ''
        @st.should_not be_valid
      end
      it '51文字以上なら失敗する' do
        @st.classname = 'a'*51
        @st.should_not be_valid
      end
    end
    context 'captionを検証するとき' do
      before do
        @st = FactoryGirl.build :speech_balloon_template
      end
      it 'テストデータの確認' do
        @st.caption = 'abcdefghi0abcdefghi0abcdefghi0'
        @st.should be_valid
      end
      it 'nullなら失敗する' do
        @st.caption = ''
        @st.should_not be_valid
      end
      it '31文字以上なら失敗する' do
        @st.caption = 'a'*31
        @st.should_not be_valid
      end
    end
    context 'tを検証するとき' do
      before do
        @st = FactoryGirl.build :speech_balloon_template
      end
      it 'テストデータの確認' do
        @st.t = 0
        @st.should be_valid
      end
      it 'nullなら失敗する' do
        @st.t = nil
        @st.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @st.t = 'a'
        @st.should_not be_valid
      end
      it '負なら失敗する' do
        @st.t = -1
        @st.should_not be_valid
      end
    end
    context 'settingsを検証するとき' do
      before do
        @st = FactoryGirl.build :speech_balloon_template
      end
      it 'テストデータの確認' do
        @st.settings = '{}'
        @st.should be_valid
      end
      it 'nullなら失敗する' do
        @st.settings = nil
        @st.should_not be_valid
      end
    end
  end
  
  describe 'データ補充に於いて' do
    before do
    end
    
  end
  
  describe 'Json解析に於いて' do
    before do
    end
    context 'テキストを渡されたとき' do
      it 'Json解析している' do
        JSON.should_receive(:parse).with(@t).exactly(1)
        r = SpeechBalloonTemplate.parse @t
      end
      it 'Hashで返す' do
        r = SpeechBalloonTemplate.parse @t
        r.is_a?(Hash).should be_true
      end
    end
    context 'パース失敗したとき' do
      it 'Falseを返す' do
        JSON.should_receive(:parse).with(any_args).and_raise('StandardError')
        r = SpeechBalloonTemplate.parse @t
        r.should be_false
      end
    end
  end
  
  describe 'システム画像置換に於いて' do
    before do
      @sp = FactoryGirl.create :system_picture
      @sp2 = FactoryGirl.create :system_picture
      @st = FactoryGirl.build :speech_balloon_template
      @hash = {"classname" => 'sore', "caption" => 'store test',
        "templates" => {
          "one" => {"balloon" => {"system_picture" => 'BASE64'}}
        }
      }
      @hashs = {"classname" => 'sore2', "caption" => 'store test2',
        "templates" => {
          "one" => {"balloon" => {"system_picture" => 'DUAL'}},
          "two" => {"balloon" => {"system_picture" => 'BASE64ENC'}}
        }
      }
    end
    context '事前チェック' do
      it 'システム画像に保存を依頼している' do
        SystemPicture.stub(:store).with(any_args).and_return(@sp)
        SystemPicture.should_receive(:store).exactly(1)
        r = @st.pic_rehash @hash
      end
    end
    context 'つつがなく終わるとき' do
      before do
        SystemPicture.stub(:store).with(any_args).and_return(@sp)
      end
      it 'システム画像のidを作成している' do
        r = @st.pic_rehash @hash
        @hash["templates"]["one"]["balloon"].has_key?("system_picture_id").should be_true
        @hash["templates"]["one"]["balloon"]["system_picture_id"].should eq @sp.id
      end
      it 'システム画像のテキストデータを削除している' do
        r = @st.pic_rehash @hash
        @hash["templates"]["one"]["balloon"].has_key?("system_picture").should_not be_true
      end
      it 'trueを返す' do
        r = @st.pic_rehash @hash
        r.should be_true
      end
    end
    context 'システム画像の作成に失敗したとき' do
      before do
        SystemPicture.stub(:store).with(any_args).and_return(false)
      end
      it 'Falseを返す' do
        r = @st.pic_rehash @hash
        r.should be_false
      end
    end
    context '複数のとき' do
      before do
        SystemPicture.stub(:store).with(any_args).and_return(@sp)
      end
      it 'システム画像のidを作成している' do
        r = @st.pic_rehash @hashs
        @hashs["templates"]["one"]["balloon"].has_key?("system_picture_id").should be_true
        @hashs["templates"]["one"]["balloon"]["system_picture_id"].should eq @sp.id
        @hashs["templates"]["two"]["balloon"].has_key?("system_picture_id").should be_true
        @hashs["templates"]["two"]["balloon"]["system_picture_id"].should eq @sp.id
      end
      it 'システム画像のテキストデータを削除している' do
        r = @st.pic_rehash @hashs
        @hashs["templates"]["one"]["balloon"].has_key?("system_picture").should_not be_true
        @hashs["templates"]["two"]["balloon"].has_key?("system_picture").should_not be_true
      end
    end
    context '例外データのとき' do
      it 'templatesがなくても何ごともなく続行する' do
        r = @st.pic_rehash({})
        r.should be_true
      end
      it 'templatesが空でも何ごともなく続行する' do
        r = @st.pic_rehash( {"templates" => nil})
        r.should be_true
      end
      it 'balloonがなくても何ごともなく続行する' do
        r = @st.pic_rehash({"templates" => {
          "one" => {}
        }})
        r.should be_true
      end
      it 'balloonが空でも何ごともなく続行する' do
        r = @st.pic_rehash({"templates" => {
          "one" => {"balloon" => nil}
        }})
        r.should be_true
      end
      it 'system_pictureがなくても何ごともなく続行する' do
        r = @st.pic_rehash({"templates" => {
          "one" => {"balloon" => {}}
        }})
        r.should be_true
      end
    end
  end
  
  describe 'フキダシテンプレート保存に於いて' do
    before do
      @st = FactoryGirl.build :speech_balloon_template
      @hash = {"classname" => 'sore', "caption" => 'store test'}
    end
    context '事前チェック' do
      it 'システム画像置換を依頼している' do
        SpeechBalloonTemplate.any_instance.stub(:pic_rehash).with(@hash).and_return(true)
        SpeechBalloonTemplate.any_instance.should_receive(:pic_rehash).exactly(1)
        r = @st.store @hash
      end
    end
    context 'つつがなく終わるとき' do
      before do
        SpeechBalloonTemplate.any_instance.stub(:pic_rehash).with(@hash).and_return(true)
      end
      it 'hashから取り出したclassnameをセットしている' do
        @st.store @hash
        @st.classname.should eq @hash["classname"]
      end
      it 'hashから取り出したcaptionをセットしている' do
        @st.store @hash
        @st.caption.should eq @hash["caption"]
      end
      it 'hashをテキスト化してsettingsにセットしている' do
        @st.store @hash
        @st.settings.should eq @hash.to_s
      end
      it 'フキダシテンプレートが作成されている' do
        lambda {
          @st.store @hash
        }.should change SpeechBalloonTemplate, :count
      end
    end
    context 'システム画像置換が失敗したとき' do
      before do
        SpeechBalloonTemplate.any_instance.stub(:pic_rehash).with(@hash).and_return(false)
      end
      it 'Falseを返す' do
        r = @st.store @hash
        r.should be_false
      end
    end
    context '保存が失敗したとき' do
      before do
        SpeechBalloonTemplate.any_instance.stub(:pic_rehash).with(@hash).and_return(true)
        SpeechBalloonTemplate.any_instance.stub(:save).with(any_args).and_return(false)
      end
      it 'Falseを返す' do
        r = @st.store @hash
        r.should be_false
      end
    end
  end
  
  describe 'フキダシテンプレート更新に於いて' do
    before do
      @st = FactoryGirl.create :speech_balloon_template
      @hash = {}
    end
    context 'つつがなく終わるとき' do
      it '保存依頼している' do
        SpeechBalloonTemplate.any_instance.stub(:store).with(@hash).and_return(true)
        SpeechBalloonTemplate.should_receive(:store).exactly(1)
        SpeechBalloonTemplate.store @st.name, @hash
      end
      context '新規のとき' do
        #新規作成をチェックしたいが、楽するために(storeが保存しないことを利用して)新規オブジェクト生成かでテスト
        it 'フキダシテンプレートは新規オブジェクトである' do
          SpeechBalloonTemplate.any_instance.stub(:store).with(@hash).and_return(true)
          r = SpeechBalloonTemplate.store :newsbtname, @hash
          r.should be_a_new SpeechBalloonTemplate
        end
      end
      context '更新のとき' do
        it '名前が一致するテンプレを返す' do
          SpeechBalloonTemplate.any_instance.stub(:store).with(@hash).and_return(true)
          r = SpeechBalloonTemplate.store @st.name, @hash
          r.name.should eq @st.name
          r.should_not be_a_new SpeechBalloonTemplate
        end
      end
    end
    context 'フキダシテンプレートの作成に失敗するとき' do
      before do
        SpeechBalloonTemplate.any_instance.stub(:store).with(any_args).and_return(false)
      end
      #成功失敗に関わらずオブジェクトを返すのでテストのやりようがない
      
    end
  end
  
  describe 'テキスト取り込みに於いて' do
    #成功でTrue、パース失敗でFalse、失敗は保存エラーのモデルを配列で返す
    before do
    end
    context 'つつがなく終わるとき' do
      it 'Json解析を依頼する' do
        SpeechBalloonTemplate.should_receive(:parse).with(any_args).exactly(1)
        SpeechBalloonTemplate.stub(:parse).with(any_args).and_return(@j)
        SpeechBalloonTemplate.import(@t)
      end
      it 'フキダシテンプレート更新を一回依頼する' do
        SpeechBalloonTemplate.stub(:store).with(any_args).and_return(SpeechBalloonTemplate.new)
        SpeechBalloonTemplate.any_instance.stub(:valid?).with(any_args).and_return(true)
        SpeechBalloonTemplate.should_receive(:store).with(any_args).exactly(1)
        SpeechBalloonTemplate.import(@t)
      end
      it 'フキダシテンプレートが追加される' do
        lambda {
          SpeechBalloonTemplate.import(@t)
        }.should change SpeechBalloonTemplate, :count
      end
      it '[]を返す' do
        SpeechBalloonTemplate.import(@t).should eq []
      end
    end
    context '複数データがつつがなく終わるとき' do
      it 'フキダシテンプレート更新を二回依頼する' do
        SpeechBalloonTemplate.stub(:store).with(any_args).and_return(SpeechBalloonTemplate.new)
        SpeechBalloonTemplate.any_instance.stub(:valid?).with(any_args).and_return(true)
        SpeechBalloonTemplate.should_receive(:store).with(any_args).exactly(2)
        SpeechBalloonTemplate.import(@ts)
      end
      it 'フキダシテンプレートが二個追加される' do
        lambda {
          SpeechBalloonTemplate.import(@ts)
        }.should change(SpeechBalloonTemplate, :count).by 2
      end
      it '[]を返す' do
        SpeechBalloonTemplate.import(@ts).should eq []
      end
    end
    #例外ケース
    context 'Json解析に失敗したとき' do
      before do
        SpeechBalloonTemplate.stub(:parse).with(any_args).and_return(false)
      end
      it 'フキダシテンプレートの数に変化がない' do
        lambda {
          SpeechBalloonTemplate.import(@t)
        }.should_not change SpeechBalloonTemplate, :count
      end
      it 'Falseを返す' do
        SpeechBalloonTemplate.import(@t).should be_false
      end
    end
    context 'フキダシテンプレート作成に失敗したとき' do
      before do
        SpeechBalloonTemplate.any_instance.stub(:save).with(any_args).and_return(false)
        SpeechBalloonTemplate.any_instance.stub(:valid?).with(any_args).and_return(false)
      end
      it 'フキダシテンプレートの数に変化がない' do
        lambda {
          SpeechBalloonTemplate.import(@t)
        }.should_not change SpeechBalloonTemplate, :count
      end
      it '配列を返す' do
        r = SpeechBalloonTemplate.import(@t)
        r.is_a?(Array).should be_true
      end
      it '配列の中身は一件' do
        r = SpeechBalloonTemplate.import(@t)
        r.should have(1).items
      end
      it 'フキダシテンプレートオブジェクトが入っている' do
        r = SpeechBalloonTemplate.import(@t)
        r.first.is_a?(SpeechBalloonTemplate).should be_true
      end
    end
    context '複数のフキダシテンプレート作成に失敗したとき' do
      #三件中、二件の失敗、一件を成功させ、成功データは戻り値に含まないことを確認する
      it 'フキダシテンプレートの数に変化がない' do
        lambda {
          SpeechBalloonTemplate.import(@tes)
        }.should_not change SpeechBalloonTemplate, :count
      end
      it '途中で保存に失敗しても全件更新依頼する' do
        SpeechBalloonTemplate.stub(:store).with(any_args).and_return(SpeechBalloonTemplate.new)
        SpeechBalloonTemplate.should_receive(:store).with(any_args).exactly(3)
        SpeechBalloonTemplate.import(@tes)
      end
      it '配列を返す' do
        r = SpeechBalloonTemplate.import(@tes)
        r.is_a?(Array).should be_true
      end
      it '配列の中身は2件' do
        r = SpeechBalloonTemplate.import(@tes)
        r.should have(2).items
      end
      it '配列の中身は失敗したフキダシテンプレートオブジェクトが入っている' do
        r = SpeechBalloonTemplate.import(@tes)
        r[0].is_a?(SpeechBalloonTemplate).should be_true
        r[0]["name"].should eq 'squareR@pettan.com'
        r[1].is_a?(SpeechBalloonTemplate).should be_true
        r[1]["name"].should eq 'squareRR@pettan.com'
      end
    end
  end
  
  describe 'インポートエラーの表示に於いて' do
  end
  
  describe 'ファイル取り込みに於いて' do
    before do
      SpeechBalloonTemplate.stub(:import).with(any_args).and_return(true)
    end
    context 'つつがなく終わるとき' do
      before do
        SpeechBalloonTemplate.stub(:import).with(any_args).and_return(true)
      end
      it 'ファイルを開いてテキストを読む' do
        File.should_receive(:open).with(@f, 'r').exactly(1)
        SpeechBalloonTemplate.import_file(@f)
      end
      it 'テキスト取り込みを依頼する' do
        SpeechBalloonTemplate.should_receive(:import).with(any_args).exactly(1)
        SpeechBalloonTemplate.import_file(@f)
      end
      #テキスト取り込み成功でTrueが返る
      it 'Trueを返す' do
        SpeechBalloonTemplate.import_file(@f).should be_true
      end
    end
    context 'ファイルが開けないとき' do
      before do
        File.stub(:open).with(any_args).and_raise('StandardError')
      end
      it 'ファイルエラーのメッセージを出力する' do
        pending
      end
      it 'Falseを返す' do
        SpeechBalloonTemplate.import_file(@f).should be_false
      end
    end
    #失敗したときは、失敗したフキダシテンプレートが配列で返る
    context 'テキスト取り込みが失敗したとき' do
      before do
        SpeechBalloonTemplate.stub(:import).with(any_args).and_return(false)
      end
      it '各フキダシテンプレートのエラーメッセージを出力する' do
        pending
      end
      it 'Falseを返す' do
        SpeechBalloonTemplate.import_file(@f).should be_false
      end
    end
  end
  
end
